@@ -1,7 +1,7 @@
-/** @file autoptr.h
- * @brief Wrapper around standard <autoptr> header.
+/** @file
+ * @brief Wrapper around standard unique_ptr template.
  */
-/* Copyright (C) 2009 Olly Betts
+/* Copyright (C) 2009,2011,2014,2015 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,12 +18,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-// We used to provide an implementation of auto_ptr called AutoPtr to work
-// around deficiencies in the implementation with some compiler, possibly
-// GCC 2.95 (it was added when that was the current GCC release).  To allow
-// us to test the water and ensure this is no longer required, we aren't
-// renaming all occurrences just yet...
+// This wrapper now only exists to make it easier to backport patches to 1.2.x
+// where AutoPtr is defined to std::auto_ptr.  This wrapper can be dropped when
+// 1.2.x is retired, or once we get to the stage of only backporting fixes for
+// serious bugs.
 #ifndef AutoPtr
 # include <memory>
-# define AutoPtr std::auto_ptr
+# define AutoPtr std::unique_ptr
 #endif
