@@ -44,8 +44,6 @@
 # include "parseint.h"
 #endif
 
-using namespace std;
-
 static int
 get_maxfd() {
 #ifdef F_MAXFD
@@ -164,7 +162,7 @@ Xapian::Internal::closefrom(int fd)
     if (dir >= 0) {
 	gdea_type base = 0;
 	struct attrlist alist;
-	memset(&alist, 0, sizeof(alist));
+	std::memset(&alist, 0, sizeof(alist));
 	alist.bitmapcount = ATTR_BIT_MAP_COUNT;
 	alist.commonattr = ATTR_CMN_NAME;
 	while (true) {
