@@ -12,12 +12,6 @@
 
 static void generate(struct generator * g, struct node * p);
 
-enum special_labels {
-
-    x_return = -1
-
-};
-
 static int new_label(struct generator * g) {
     return g->next_label++;
 }
@@ -43,25 +37,38 @@ static void wi3(struct generator * g, int i) {
 
 static void write_varname(struct generator * g, struct name * p) {
 
-    int ch = "SBIrxg"[p->type];
+    int ch = "SIIrxg"[p->type];
     switch (p->type) {
         case t_external:
             write_string(g, g->options->externals_prefix); break;
         case t_string:
         case t_boolean:
-        case t_integer:
-	    if (g->options->make_lang == LANG_C) {
-		write_char(g, ch);
-		write_char(g, '[');
-		write_int(g, p->count);
-		write_char(g, ']');
-		return;
-	    }
-	    /* FALLTHRU */
+        case t_integer: {
+            if (g->options->make_lang != LANG_C) goto use_name;
+            int count = p->count;
+            if (count < 0) {
+                fprintf(stderr, "Reference to optimised out variable ");
+                report_b(stderr, p->b);
+                fprintf(stderr, " attempted\n");
+                exit(1);
+            }
+            if (p->type == t_boolean) {
+                /* We use a single array for booleans and integers, with the
+                 * integers first.
+                 */
+                count += g->analyser->name_count[t_integer];
+            }
+            write_char(g, ch);
+            write_char(g, '[');
+            write_int(g, count);
+            write_char(g, ']');
+            return;
+        }
         default:
+use_name:
             write_char(g, ch); write_char(g, '_');
     }
-    str_append_b(g->outbuf, p->b);
+    write_b(g, p->b);
 }
 
 static void write_varref(struct generator * g, struct name * p) {  /* reference to variable */
@@ -88,7 +95,7 @@ static void wlitch(struct generator * g, int ch) {
         }
         write_char(g, ch);
         write_char(g, '\'');
-    }  else {
+    } else {
         write_string(g, "0x"); write_hex(g, ch);
     }
 }
@@ -106,7 +113,9 @@ static void wlitarray(struct generator * g, symbol * p) {  /* write literal arra
 
 static void wlitref(struct generator * g, symbol * p) {  /* write ref to literal array */
 
-    if (SIZE(p) == 0) write_char(g, '0'); else {
+    if (SIZE(p) == 0) {
+        write_char(g, '0');
+    } else {
         struct str * s = g->outbuf;
         g->outbuf = g->declarations;
         write_string(g, "static const symbol s_"); write_int(g, g->literalstring_count); write_string(g, "[] = { ");
@@ -123,37 +132,48 @@ static void write_margin(struct generator * g) {
     for (i = 0; i < g->margin; i++) write_string(g, "    ");
 }
 
-static void write_comment(struct generator * g, struct node * p) {
-
-    ws_opt_space(g, "/* ");
+void write_comment_content(struct generator * g, struct node * p) {
     switch (p->type) {
         case c_mathassign:
         case c_plusassign:
         case c_minusassign:
         case c_multiplyassign:
         case c_divideassign:
+            if (p->name) {
+                write_char(g, '$');
+                write_b(g, p->name->b);
+                write_char(g, ' ');
+            }
+            write_string(g, name_of_token(p->type));
+            write_string(g, " <integer expression>");
+            break;
         case c_eq:
         case c_ne:
         case c_gr:
         case c_ge:
         case c_ls:
         case c_le:
-            if (p->name) {
-                write_char(g, '$');
-                str_append_b(g->outbuf, p->name->b);
-                write_char(g, ' ');
-            }
+            write_string(g, "$(<integer expression> ");
             write_string(g, name_of_token(p->type));
-            write_string(g, " <integer expression>");
+            write_string(g, " <integer expression>)");
             break;
         default:
             write_string(g, name_of_token(p->type));
             if (p->name) {
                 write_char(g, ' ');
-                str_append_b(g->outbuf, p->name->b);
+                write_b(g, p->name->b);
             }
     }
-    write_string(g, ", line "); write_int(g, p->line_number); write_string(g, " */");
+    write_string(g, ", line ");
+    write_int(g, p->line_number);
+}
+
+static void write_comment(struct generator * g, struct node * p) {
+    if (g->options->comments) {
+        ws_opt_space(g, "/* ");
+        write_comment_content(g, p);
+        write_string(g, " */");
+    }
     write_newline(g);
 }
 
@@ -181,7 +201,7 @@ static void wk(struct generator * g, struct node * p, int keep_limit) {
         write_string(g, "int c");
         write_int(g, g->keep_count);
         w(g, " = ~zc");
-	if (keep_limit) {
+        if (keep_limit) {
             write_string(g, ", mlimit");
             write_int(g, g->keep_count);
         }
@@ -190,7 +210,7 @@ static void wk(struct generator * g, struct node * p, int keep_limit) {
         write_string(g, "int m");
         write_int(g, g->keep_count);
         w(g, " = ~zl - ~zc");
-	if (keep_limit) {
+        if (keep_limit) {
             write_string(g, ", mlimit");
             write_int(g, g->keep_count);
         }
@@ -245,8 +265,7 @@ static void write_failure(struct generator * g, struct node * p) {          /* f
         }
         write_char(g, ' ');
     }
-    switch (g->failure_label)
-    {
+    switch (g->failure_label) {
         case x_return:
             write_string(g, "return 0;");
             break;
@@ -687,13 +706,20 @@ static void generate_do(struct generator * g, struct node * p) {
         writef(g, "~M~C", p);
     }
 
-    g->failure_label = new_label(g);
-    g->label_used = 0;
-    g->failure_keep_count = 0;
-    generate(g, p->left);
+    if (p->left->type == c_call) {
+        /* Optimise do <call> */
+        g->V[0] = p->left->name;
+        writef(g, "~{int ret = ~V0(~z);~C", p->left);
+        w(g, "~Mif (ret < 0) return ret;~N~}");
+    } else {
+        g->failure_label = new_label(g);
+        g->label_used = 0;
+        g->failure_keep_count = 0;
+        generate(g, p->left);
 
-    if (g->label_used)
-        wsetl(g, g->failure_label);
+        if (g->label_used)
+            wsetl(g, g->failure_label);
+    }
     if (keep_c) {
         w(g, "~M"); wrestore(g, p, keep_c);
         w(g, "~N~}");
@@ -725,16 +751,16 @@ static void generate_GO_grouping(struct generator * g, struct node * p, int is_g
     g->I[0] = q->smallest_ch;
     g->I[1] = q->largest_ch;
     if (is_goto) {
-        writef(g, "~Mif (~S1_grouping~S0~S2(~Z~V0, ~I0, ~I1, 1) < 0) ~f /* goto */~C", p);
+        writef(g, "~Mif (~S1_grouping~S0~S2(~Z~V0, ~I0, ~I1, 1) < 0) ~f~C", p);
     } else {
-        writef(g, "~{ /* gopast */~C"
-	      "~Mint ret = ~S1_grouping~S0~S2(~Z~V0, ~I0, ~I1, 1);~N"
-	      "~Mif (ret < 0) ~f~N", p);
-	if (p->mode == m_forward)
-	    w(g, "~M~zc += ret;~N");
-	else
-	    w(g, "~M~zc -= ret;~N");
-	w(g, "~}");
+        writef(g, "~{~C"
+              "~Mint ret = ~S1_grouping~S0~S2(~Z~V0, ~I0, ~I1, 1);~N"
+              "~Mif (ret < 0) ~f~N", p);
+        if (p->mode == m_forward)
+            w(g, "~M~zc += ret;~N");
+        else
+            w(g, "~M~zc -= ret;~N");
+        w(g, "~}");
     }
 }
 
@@ -746,14 +772,17 @@ static void generate_GO(struct generator * g, struct node * p, int style) {
     int a1 = g->failure_keep_count;
 
     if (p->left->type == c_grouping || p->left->type == c_non) {
-	/* Special case for "goto" or "gopast" when used on a grouping or an
-	 * inverted grouping - the movement of c by the matching action is
-	 * exactly what we want! */
+        /* Special case for "goto" or "gopast" when used on a grouping or an
+         * inverted grouping - the movement of c by the matching action is
+         * exactly what we want! */
 #ifdef OPTIMISATION_WARNINGS
-	printf("Optimising %s %s\n", style ? "goto" : "gopast", p->left->type == c_non ? "non" : "grouping");
+        printf("Optimising %s %s\n", style ? "goto" : "gopast", p->left->type == c_non ? "non" : "grouping");
 #endif
-	generate_GO_grouping(g, p->left, style, p->left->type == c_non);
-	return;
+        if (g->options->comments) {
+            writef(g, "~M~C", p);
+        }
+        generate_GO_grouping(g, p->left, style, p->left->type == c_non);
+        return;
     }
 
     w(g, "~Mwhile(1) {"); writef(g, "~C~+", p);
@@ -798,9 +827,13 @@ static void generate_loop(struct generator * g, struct node * p) {
          "~}");
 }
 
-static void generate_repeat(struct generator * g, struct node * p, int atleast_case) {
+static void generate_repeat_or_atleast(struct generator * g, struct node * p, int atleast_case) {
     int keep_c = 0;
-    writef(g, "~Mwhile(1) {~C~+", p);
+    if (atleast_case) {
+        writef(g, "~Mwhile(1) {~+~N", p);
+    } else {
+        writef(g, "~Mwhile(1) {~+~C", p);
+    }
 
     if (repeat_restore(g, p->left)) {
         writef(g, "~M~k~N", p);
@@ -826,14 +859,18 @@ static void generate_repeat(struct generator * g, struct node * p, int atleast_c
       "~}");
 }
 
+static void generate_repeat(struct generator * g, struct node * p) {
+    generate_repeat_or_atleast(g, p, false);
+}
+
 static void generate_atleast(struct generator * g, struct node * p) {
-    w(g, "~{int i = "); generate_AE(g, p->AE); w(g, ";~N");
+    w(g, "~{int i = "); generate_AE(g, p->AE); w(g, ";~C");
     {
         int used = g->label_used;
         int a0 = g->failure_label;
         int a1 = g->failure_keep_count;
 
-        generate_repeat(g, p, true);
+        generate_repeat_or_atleast(g, p, true);
 
         g->label_used = used;
         g->failure_label = a0;
@@ -863,7 +900,7 @@ static void generate_atmark(struct generator * g, struct node * p) {
 static void generate_hop(struct generator * g, struct node * p) {
     g->S[0] = p->mode == m_forward ? "+" : "-";
     g->S[1] = p->mode == m_forward ? "0" :
-	(g->options->make_lang == LANG_C ? "z->lb" : "lb");
+        (g->options->make_lang == LANG_C ? "z->lb" : "lb");
     if (g->options->encoding == ENC_UTF8) {
         w(g, "~{int ret = skip_utf8(~zp, ~zc, ~S1, ~zl, ~S0 ");
         generate_AE(g, p->AE); writef(g, ");~C", p);
@@ -878,13 +915,9 @@ static void generate_hop(struct generator * g, struct node * p) {
 }
 
 static void generate_delete(struct generator * g, struct node * p) {
-#if 1
-    writef(g, "~Mif (slice_del(~Z) == -1) return -1;~C", p);
-#else
     writef(g, "~{int ret = slice_del(~Z);~C", p);
     writef(g, "~Mif (ret < 0) return ret;~N"
           "~}", p);
-#endif
 }
 
 static void generate_tolimit(struct generator * g, struct node * p) {
@@ -919,8 +952,8 @@ static void generate_sliceto(struct generator * g, struct node * p) {
     g->V[0] = p->name;
     writef(g, "~{symbol * ret = slice_to(~Z~V0);~C"
           "~Mif (ret == 0) return -1;~N"
-	  "~M~V0 = ret;~N"
-	  "~}", p);
+          "~M~V0 = ret;~N"
+          "~}", p);
 }
 
 static void generate_insert(struct generator * g, struct node * p, int style) {
@@ -928,30 +961,30 @@ static void generate_insert(struct generator * g, struct node * p, int style) {
     int keep_c = style == c_attach;
     if (p->mode == m_backward) keep_c = !keep_c;
     if (g->options->make_lang == LANG_C)
-	writef(g, "~{int ret;~N", p);
+        writef(g, "~{int ret;~N", p);
     if (keep_c) w(g, "~{int saved_c = ~zc;~N");
     if (g->options->make_lang == LANG_C)
-	writef(g, "~Mret = insert_~$(~Z~zc, ~zc, ~a);~C", p);
+        writef(g, "~Mret = insert_~$(~Z~zc, ~zc, ~a);~C", p);
     else
-	writef(g, "~Minsert_~$(~Z~zc, ~zc, ~a);~C", p);
+        writef(g, "~Minsert_~$(~Z~zc, ~zc, ~a);~C", p);
     if (keep_c) w(g, "~M~zc = saved_c;~N~}");
     if (g->options->make_lang == LANG_C)
-	writef(g, "~Mif (ret < 0) return ret;~N"
-	      "~}", p);
+        writef(g, "~Mif (ret < 0) return ret;~N"
+              "~}", p);
 }
 
 static void generate_assignfrom(struct generator * g, struct node * p) {
 
     int keep_c = p->mode == m_forward; /* like 'attach' */
     if (g->options->make_lang == LANG_C)
-	writef(g, "~{int ret;~N", p);
+        writef(g, "~{int ret;~N", p);
     if (keep_c) writef(g, "~{int saved_c = ~zc;~N", p);
     w(g, g->options->make_lang == LANG_C ? "~Mret =" : "~M");
     writef(g, keep_c ? "insert_~$(~Z~zc, ~zl, ~a);~C" : "insert_~$(~Z~zlb, ~zc, ~a);~C", p);
     if (keep_c) w(g, "~M~zc = saved_c;~N~}");
     if (g->options->make_lang == LANG_C)
-	writef(g, "~Mif (ret < 0) return ret;~N"
-	      "~}", p);
+        writef(g, "~Mif (ret < 0) return ret;~N"
+              "~}", p);
 }
 
 /* bugs marked <======= fixed 22/7/02. Similar fixes required for Java */
@@ -966,7 +999,7 @@ static void generate_slicefrom(struct generator * g, struct node * p) {
 
 static void generate_setlimit(struct generator * g, struct node * p) {
     int keep_c;
-    if (p->left && p->left->type == c_tomark && !p->left->right) {
+    if (p->left && p->left->type == c_tomark) {
         /* Special case for:
          *
          *   setlimit tomark AE for C
@@ -1034,22 +1067,24 @@ static void generate_dollar(struct generator * g, struct node * p) {
     keep_token = ++g->keep_count;
     g->I[0] = keep_token;
     if (g->options->make_lang != LANG_C) {
-	const char ** var;
-	writef(g, "~{~C", p);
-	for (var = vars; *var; ++var) {
-	    g->S[0] = *var;
-	    w(g, "~Mint ~S0~I0 = ~S0;~N");
-	}
+        const char ** var;
+        writef(g, "~{~C", p);
+        for (var = vars; *var; ++var) {
+            g->S[0] = *var;
+            w(g, "~Mint ~S0~I0 = ~S0;~N");
+        }
     } else {
-	writef(g, "~{struct SN_env env~I0 = * z;~C", p);
+        writef(g, "~{struct SN_env env~I0 = * z;~C", p);
     }
     g->V[0] = p->name;
-    writef(g, "~Mint failure = 1; /* assume failure */~N"
+    /* Assume failure. */
+    writef(g, "~Mint failure = 1;~N"
           "~M~zp = ~V0;~N"
           "~M~zlb = ~zc = 0;~N"
           "~M~zl = SIZE(~zp);~N", p);
     generate(g, p->left);
-    w(g, "~Mfailure = 0; /* mark success */~N");
+    /* Mark success. */
+    w(g, "~Mfailure = 0;~N");
     if (g->label_used)
         wsetl(g, g->failure_label);
     g->V[0] = p->name; /* necessary */
@@ -1060,17 +1095,17 @@ static void generate_dollar(struct generator * g, struct node * p) {
 
     g->I[0] = keep_token;
     if (g->options->make_lang != LANG_C) {
-	const char ** var;
-	w(g, "~M~V0 = ~zp;~N");
-	for (var = vars; *var; ++var) {
-	    g->S[0] = *var;
-	    w(g, "~M~S0 = ~S0~I0;~N");
-	}
-	writef(g, "~Mif (failure) ~f~N~}", p);
+        const char ** var;
+        w(g, "~M~V0 = ~zp;~N");
+        for (var = vars; *var; ++var) {
+            g->S[0] = *var;
+            w(g, "~M~S0 = ~S0~I0;~N");
+        }
+        writef(g, "~Mif (failure) ~f~N~}", p);
     } else {
-	writef(g, "~M~V0 = z->p;~N"
-	      "~M* z = env~I0;~N"
-	      "~Mif (failure) ~f~N~}", p);
+        writef(g, "~M~V0 = z->p;~N"
+              "~M* z = env~I0;~N"
+              "~Mif (failure) ~f~N~}", p);
     }
 }
 
@@ -1083,9 +1118,13 @@ static void generate_integer_assign(struct generator * g, struct node * p, char
 
 static void generate_integer_test(struct generator * g, struct node * p, char * s) {
 
-    g->V[0] = p->name;
-    g->S[0] = s;
-    w(g, "~Mif (!(~V0 ~S0 "); generate_AE(g, p->AE); writef(g, ")) ~f~C", p);
+    w(g, "~Mif (!(");
+    generate_AE(g, p->left);
+    write_char(g, ' ');
+    write_string(g, s);
+    write_char(g, ' ');
+    generate_AE(g, p->AE);
+    writef(g, ")) ~f~C", p);
 }
 
 static void generate_call(struct generator * g, struct node * p) {
@@ -1155,16 +1194,18 @@ static void generate_define(struct generator * g, struct node * p) {
     g->next_label = 0;
 
     if (g->options->make_lang == LANG_C)
-	g->S[0] = q->type == t_routine ? "static" : "extern";
+        g->S[0] = q->type == t_routine ? "static" : "extern";
     else
-	g->S[0] = g->options->name;
+        g->S[0] = g->options->name;
     g->V[0] = q;
 
     if (g->options->make_lang == LANG_C)
-	w(g, "~N~S0 int ~V0(struct SN_env * z) {");
+        w(g, "~N~S0 int ~V0(struct SN_env * z) {");
     else
-	w(g, "~Nint Xapian::~S0::~V0() {");
-    write_string(g, p->mode == m_forward ? " /* forwardmode */" : " /* backwardmode */");
+        w(g, "~Nint Xapian::~S0::~V0() {");
+    if (g->options->comments) {
+        write_string(g, p->mode == m_forward ? " /* forwardmode */" : " /* backwardmode */");
+    }
     w(g, "~N~+");
     if (p->amongvar_needed) w(g, "~Mint among_var;~N");
     g->failure_keep_count = 0;
@@ -1298,8 +1339,8 @@ static void generate_substring(struct generator * g, struct node * p) {
 #endif
     }
 
-    if (x->command_count == 0 && x->starter == 0) {
-        w(g, "~Mif (!(find_among~S0(s_pool, ~Za_~I0, ~I1, ");
+    if (!x->amongvar_needed) {
+        writef(g, "~Mif (!(find_among~S0(s_pool, ~Za_~I0, ~I1, ", p);
         if (x->function_count) {
             w(g, "af_~I0, af");
         } else {
@@ -1323,27 +1364,25 @@ static void generate_substring(struct generator * g, struct node * p) {
 static void generate_among(struct generator * g, struct node * p) {
 
     struct among * x = p->among;
-    int case_number = 1;
 
     if (x->substring == 0) generate_substring(g, p);
-    if (x->command_count == 0 && x->starter == 0) return;
 
     if (x->starter != 0) generate(g, x->starter);
 
-    writef(g, "~Mswitch (among_var) {~C~+"
-              "~Mcase 0: ~f~N", p);
-
-    p = p->left;
-    if (p != 0 && p->type != c_literalstring) p = p->right;
-
-    while (p) {
-        if (p->type == c_bra && p->left != 0) {
-            g->I[0] = case_number++;
-            w(g, "~Mcase ~I0:~N~+"); generate(g, p); w(g, "~Mbreak;~N~-");
+    if (x->command_count == 1 && x->nocommand_count == 0) {
+        /* Only one outcome ("no match" already handled). */
+        generate(g, x->commands[0]);
+    } else if (x->command_count > 0) {
+        int i;
+        writef(g, "~Mswitch (among_var) {~C~+", p);
+        for (i = 1; i <= x->command_count; i++) {
+            g->I[0] = i;
+            w(g, "~Mcase ~I0:~N~+");
+            generate(g, x->commands[i - 1]);
+            w(g, "~Mbreak;~N~-");
         }
-        p = p->right;
+        w(g, "~}");
     }
-    w(g, "~}");
 }
 
 static void generate_booltest(struct generator * g, struct node * p) {
@@ -1371,8 +1410,7 @@ static void generate(struct generator * g, struct node * p) {
     int a0 = g->failure_label;
     int a1 = g->failure_keep_count;
 
-    switch (p->type)
-    {
+    switch (p->type) {
         case c_define:        generate_define(g, p); break;
         case c_bra:           generate_bra(g, p); break;
         case c_and:           generate_and(g, p); break;
@@ -1388,7 +1426,7 @@ static void generate(struct generator * g, struct node * p) {
         case c_do:            generate_do(g, p); break;
         case c_goto:          generate_GO(g, p, 1); break;
         case c_gopast:        generate_GO(g, p, 0); break;
-        case c_repeat:        generate_repeat(g, p, false); break;
+        case c_repeat:        generate_repeat(g, p); break;
         case c_loop:          generate_loop(g, p); break;
         case c_atleast:       generate_atleast(g, p); break;
         case c_setmark:       generate_setmark(g, p); break;
@@ -1441,41 +1479,47 @@ static void generate(struct generator * g, struct node * p) {
     g->failure_keep_count = a1;
 }
 
-static void generate_start_comment(struct generator * g) {
+void write_generated_comment_content(struct generator * g) {
+    w(g, "Generated by Snowball " SNOWBALL_VERSION
+         " - https://snowballstem.org/");
+}
 
-    if (g->options->make_lang == LANG_C)
-	w(g, "/* This file was generated automatically by the Snowball to ISO C compiler */~N");
-    else
-	w(g, "/* This file was generated automatically by the Snowball to ISO C++ compiler */~N");
-    w(g, "/* http://snowballstem.org/ */~N");
+void write_start_comment(struct generator * g,
+                         const char * comment_start,
+                         const char * comment_end) {
+    write_margin(g);
+    w(g, comment_start);
+    write_generated_comment_content(g);
+    if (comment_end) {
+        w(g, comment_end);
+    }
+    w(g, "~N~N");
 }
 
 static void generate_head(struct generator * g) {
 
     if (g->options->make_lang != LANG_C) {
-	const char * s = g->options->output_file;
-	const char * leaf;
-	w(g, "~N"
-	     "#include <config.h>~N"
-	     "#include <limits.h>~N");
-	if (!s) abort(); /* checked in driver.c */
-	leaf = strrchr(s, '/');
-	if (leaf) ++leaf; else leaf = s;
-	write_string(g, "#include \"");
-	write_string(g, leaf);
-	w(g, ".h\"~N~N");
-	return;
+        const char * s = g->options->output_file;
+        const char * leaf;
+        w(g, "~N"
+             "#include <config.h>~N"
+             "#include <limits.h>~N");
+        if (!s) abort(); /* checked in driver.c */
+        leaf = strrchr(s, '/');
+        if (leaf) ++leaf; else leaf = s;
+        write_string(g, "#include \"");
+        write_string(g, leaf);
+        w(g, ".h\"~N~N");
+        return;
     }
 
-    if (g->options->runtime_path == 0) {
-        w(g, "~N#include \"header.h\"~N~N");
-    } else {
-        w(g, "~N#include \"");
+    w(g, "#include \"");
+    if (g->options->runtime_path) {
         write_string(g, g->options->runtime_path);
         if (g->options->runtime_path[strlen(g->options->runtime_path) - 1] != '/')
             write_char(g, '/');
-        w(g, "header.h\"~N~N");
     }
+    w(g, "header.h\"~N~N");
 }
 
 static void generate_routine_headers(struct generator * g) {
@@ -1506,56 +1550,56 @@ static unsigned pool_size = 0;
 static void generate_among_pool(struct generator * g, struct among * x) {
 
     while (x) {
-	struct amongvec * v = x->b;
-	int i;
-	char * done = check_malloc(x->literalstring_count);
-	memset(done, 0, x->literalstring_count);
+        struct amongvec * v = x->b;
+        int i;
+        char * done = check_malloc(x->literalstring_count);
+        memset(done, 0, x->literalstring_count);
 
-	g->I[0] = x->number;
+        g->I[0] = x->number;
 
-	for (i = 0; i < x->literalstring_count; i++, v++)
-	{
-	    int j;
-	    if (v->size == 0 || done[i]) continue;
-	    g->I[1] = i;
-	    /* Eliminate entries which are just substrings of other entries */
-	    for (j = 0; j < x->literalstring_count; j++) {
-		if (j == i) continue;
-		if (v->size <= v[j - i].size) {
-		    size_t offset = v[j - i].size - v->size;
-		    size_t len = v->size * sizeof(symbol);
-		    do {
-			if (memcmp(v->b, v[j - i].b + offset, len) == 0) {
-			    g->I[2] = j;
-			    if (offset) {
-				g->I[3] = offset;
-				w(g, "#define s_~I0_~I1 (s_~I0_~I2 + ~I3)~N");
-			    } else {
-				w(g, "#define s_~I0_~I1 s_~I0_~I2~N");
-			    }
-			    goto done;
-			}
-		    } while (offset--);
-		}
-	    }
-	    if (v->size) {
-		if (pool_size == 0) {
-		    w(g, "static const symbol s_pool[] = {~N");
-		}
-		g->I[2] = pool_size;
-		w(g, "#define s_~I0_~I1 ~I2~N");
-		g->L[0] = v->b;
-		w(g, "~A0,~N");
-		pool_size += v->size;
-	    }
+        for (i = 0; i < x->literalstring_count; i++, v++)
+        {
+            int j;
+            if (v->size == 0 || done[i]) continue;
+            g->I[1] = i;
+            /* Eliminate entries which are just substrings of other entries */
+            for (j = 0; j < x->literalstring_count; j++) {
+                if (j == i) continue;
+                if (v->size <= v[j - i].size) {
+                    size_t offset = v[j - i].size - v->size;
+                    size_t len = v->size * sizeof(symbol);
+                    do {
+                        if (memcmp(v->b, v[j - i].b + offset, len) == 0) {
+                            g->I[2] = j;
+                            if (offset) {
+                                g->I[3] = offset;
+                                w(g, "#define s_~I0_~I1 (s_~I0_~I2 + ~I3)~N");
+                            } else {
+                                w(g, "#define s_~I0_~I1 s_~I0_~I2~N");
+                            }
+                            goto done;
+                        }
+                    } while (offset--);
+                }
+            }
+            if (v->size) {
+                if (pool_size == 0) {
+                    w(g, "static const symbol s_pool[] = {~N");
+                }
+                g->I[2] = pool_size;
+                w(g, "#define s_~I0_~I1 ~I2~N");
+                g->L[0] = v->b;
+                w(g, "~A0,~N");
+                pool_size += v->size;
+            }
 done: ;
-	}
+        }
 
-	check_free(done);
-	x = x->next;
+        check_free(done);
+        x = x->next;
     }
     if (pool_size != 0) {
-	w(g, "};~N~N");
+        w(g, "};~N~N");
     }
 }
 
@@ -1587,27 +1631,27 @@ static void generate_among_table(struct generator * g, struct among * x) {
     w(g, "};~N~N");
 
     if (x->function_count) {
-	g->I[1] = x->literalstring_count;
-	w(g, "~Mstatic const unsigned char af_~I0[~I1] =~N{~N");
+        g->I[1] = x->literalstring_count;
+        w(g, "~Mstatic const unsigned char af_~I0[~I1] =~N{~N");
 
-	v = x->b;
-	{
-	    int i;
-	    for (i = 0; i < x->literalstring_count; i++) {
-		g->I[1] = i;
+        v = x->b;
+        {
+            int i;
+            for (i = 0; i < x->literalstring_count; i++) {
+                g->I[1] = i;
 
-		w(g, "/*~J1 */ ");
-		if (v[i].function == 0) {
-		    w(g, "0");
-		} else {
-		    write_int(g, v[i].function->among_func_count);
-		    g->V[0] = v[i].function;
-		    w(g, " /* t~W0 */");
-		}
-		if (i < x->literalstring_count - 1) w(g, ",~N");
-	    }
+                w(g, "/*~J1 */ ");
+                if (v[i].function == 0) {
+                    w(g, "0");
+                } else {
+                    write_int(g, v[i].function->among_func_count);
+                    g->V[0] = v[i].function;
+                    w(g, " /* t~W0 */");
+                }
+                if (i < x->literalstring_count - 1) w(g, ",~N");
+            }
         }
-	w(g, "~N};~N~N");
+        w(g, "~N};~N~N");
     }
 }
 
@@ -1619,31 +1663,31 @@ static void generate_amongs(struct generator * g) {
     g->S[0] = g->options->name;
     for (q = g->analyser->names; q; q = q->next) {
         if (q->type == t_routine && q->used_in_among) {
-	    q->among_func_count = ++among_func_count;
-	    g->V[0] = q;
-	    w(g, "static int t~V0(Xapian::StemImplementation * this_ptr) {~N"
-		 "    return (static_cast<Xapian::~S0 *>(this_ptr))->~V0();~N"
-		 "}~N"
-		 "~N");
-	}
+            q->among_func_count = ++among_func_count;
+            g->V[0] = q;
+            w(g, "static int t~V0(Xapian::StemImplementation * this_ptr) {~N"
+                 "    return (static_cast<Xapian::~S0 *>(this_ptr))->~V0();~N"
+                 "}~N"
+                 "~N");
+        }
     }
 
     if (among_func_count) {
-	g->I[0] = among_func_count;
-	w(g, "~Mstatic const among_function af[~I0] =~N{~N");
+        g->I[0] = among_func_count;
+        w(g, "~Mstatic const among_function af[~I0] =~N{~N");
 
-	q = g->analyser->names;
-	g->S[0] = g->options->name;
-	for (q = g->analyser->names; q; q = q->next) {
-	    if (q->type == t_routine && q->used_in_among) {
-		g->V[0] = q;
-		g->I[0] = q->among_func_count;
-		w(g, "/*~J0 */ t~V0");
-		if (q->among_func_count < among_func_count) w(g, ",~N"); else w(g, "~N");
-	    }
-	}
+        q = g->analyser->names;
+        g->S[0] = g->options->name;
+        for (q = g->analyser->names; q; q = q->next) {
+            if (q->type == t_routine && q->used_in_among) {
+                g->V[0] = q;
+                g->I[0] = q->among_func_count;
+                w(g, "/*~J0 */ t~V0");
+                if (q->among_func_count < among_func_count) w(g, ",~N"); else w(g, "~N");
+            }
+        }
 
-	w(g, "};~N~N");
+        w(g, "};~N~N");
     }
 
     generate_among_pool(g, g->analyser->amongs);
@@ -1680,7 +1724,8 @@ static void generate_grouping_table(struct generator * g, struct grouping * q) {
 static void generate_groupings(struct generator * g) {
     struct grouping * q;
     for (q = g->analyser->groupings; q; q = q->next) {
-        generate_grouping_table(g, q);
+        if (q->name->used)
+            generate_grouping_table(g, q);
     }
 }
 
@@ -1689,41 +1734,40 @@ static void generate_create(struct generator * g) {
     int * p = g->analyser->name_count;
 
     if (g->options->make_lang != LANG_C) {
-	struct name * q = g->analyser->names;
-	int first = true;
-	const char * dtor;
-	g->S[0] = g->options->name;
-	dtor = strrchr(g->options->name, ':');
-	if (dtor) ++dtor; else dtor = g->options->name;
-	g->S[1] = dtor;
-	w(g, "~N"
-	     "Xapian::~S0::~S1()");
-	while (q) {
-	    if (q->type < t_routine) {
-		w(g, first ? "~N    : " : ", ");
-		first = false;
-		g->V[0] = q;
-		w(g, "~W0(0)");
-	    }
-	    q = q->next;
-	}
-	w(g, "~N{~N");
-	q = g->analyser->names;
-	while (q) {
-	    if (q->type == t_string) {
-		g->V[0] = q;
-		w(g, "    ~W0 = create_s();~N");
-	    }
-	    q = q->next;
-	}
-	w(g, "}~N");
+        struct name * q = g->analyser->names;
+        int first = true;
+        const char * dtor;
+        g->S[0] = g->options->name;
+        dtor = strrchr(g->options->name, ':');
+        if (dtor) ++dtor; else dtor = g->options->name;
+        g->S[1] = dtor;
+        w(g, "~N"
+             "Xapian::~S0::~S1()");
+        while (q) {
+            if (q->type < t_routine) {
+                w(g, first ? "~N    : " : ", ");
+                first = false;
+                g->V[0] = q;
+                w(g, "~W0(0)");
+            }
+            q = q->next;
+        }
+        w(g, "~N{~N");
+        q = g->analyser->names;
+        while (q) {
+            if (q->type == t_string) {
+                g->V[0] = q;
+                w(g, "    ~W0 = create_s();~N");
+            }
+            q = q->next;
+        }
+        w(g, "}~N");
 
-	return;
+        return;
     }
 
     g->I[0] = p[t_string];
-    g->I[1] = p[t_integer];
-    g->I[2] = p[t_boolean];
+    g->I[1] = p[t_integer] + p[t_boolean];
     w(g, "~N"
          "extern struct SN_env * ~pcreate_env(void) { return SN_create_env(~I0, ~I1, ~I2); }"
          "~N");
@@ -1733,35 +1777,35 @@ static void generate_close(struct generator * g) {
 
     int * p = g->analyser->name_count;
     if (g->options->make_lang != LANG_C) {
-	struct name * q = g->analyser->names;
-	const char * dtor;
-	const char * lang;
-	g->S[0] = g->options->name;
-	dtor = strrchr(g->options->name, ':');
-	if (dtor) ++dtor; else dtor = g->options->name;
-	g->S[1] = dtor;
-	lang = strrchr(g->options->output_file, '/');
-	if (lang) ++lang; else lang = g->options->output_file;
-	g->S[2] = lang;
-	w(g, "~N"
-	     "Xapian::~S0::~~~S1()~N"
-	     "{~N");
-	while (q) {
-	    if (q->type == t_string) {
-		g->V[0] = q;
-		w(g, "    lose_s(~W0);~N");
-	    }
-	    q = q->next;
-	}
-	w(g, "}~N");
+        struct name * q = g->analyser->names;
+        const char * dtor;
+        const char * lang;
+        g->S[0] = g->options->name;
+        dtor = strrchr(g->options->name, ':');
+        if (dtor) ++dtor; else dtor = g->options->name;
+        g->S[1] = dtor;
+        lang = strrchr(g->options->output_file, '/');
+        if (lang) ++lang; else lang = g->options->output_file;
+        g->S[2] = lang;
+        w(g, "~N"
+             "Xapian::~S0::~~~S1()~N"
+             "{~N");
+        while (q) {
+            if (q->type == t_string) {
+                g->V[0] = q;
+                w(g, "    lose_s(~W0);~N");
+            }
+            q = q->next;
+        }
+        w(g, "}~N");
 
-	w(g, "~N"
-	     "std::string~N"
-	     "Xapian::~S0::get_description() const~N"
-	     "{~N"
-	     "    return \"~S2\";~N"
-	     "}~N");
-	return;
+        w(g, "~N"
+             "std::string~N"
+             "Xapian::~S0::get_description() const~N"
+             "{~N"
+             "    return \"~S2\";~N"
+             "}~N");
+        return;
     }
 
     g->I[0] = p[t_string];
@@ -1782,69 +1826,69 @@ static void generate_header_file(struct generator * g) {
     g->S[0] = vp;
 
     if (g->options->make_lang != LANG_C) {
-	const char * p;
-	w(g, "~N"
-	     "#include \"steminternal.h\"~N"
-	     "~N"
-	     "namespace Xapian {~N"
-	     "~N");
+        const char * p;
+        w(g, "~N"
+             "#include \"steminternal.h\"~N"
+             "~N"
+             "namespace Xapian {~N"
+             "~N");
 
-	g->S[1] = g->options->name;
-	w(g, "class ~S1 ");
-	if (g->options->parent_class_name) {
-	    g->S[1] = g->options->parent_class_name;
-	    w(g, ": public ~S1 ");
-	}
-	w(g, "{~N");
-	for (q = g->analyser->names; q; q = q->next) {
-	    switch (q->type) {
-		case t_string:  g->S[1] = "symbol *"; goto label1;
-		case t_integer: g->S[1] = "int"; goto label1;
-		case t_boolean: g->S[1] = "unsigned char";
-		label1:
-		    g->V[0] = q;
-		    w(g, "    ~S1 ~W0;~N");
-		    break;
-	     }
-	}
+        g->S[1] = g->options->name;
+        w(g, "class ~S1 ");
+        if (g->options->parent_class_name) {
+            g->S[1] = g->options->parent_class_name;
+            w(g, ": public ~S1 ");
+        }
+        w(g, "{~N");
+        for (q = g->analyser->names; q; q = q->next) {
+            switch (q->type) {
+                case t_string:  g->S[1] = "symbol *"; goto label1;
+                case t_integer: g->S[1] = "int"; goto label1;
+                case t_boolean: g->S[1] = "unsigned char";
+                label1:
+                    g->V[0] = q;
+                    w(g, "    ~S1 ~W0;~N");
+                    break;
+             }
+        }
 
-	for (q = g->analyser->names; q; q = q->next) {
-	    if (q->type == t_routine && !q->used_in_among) {
-		g->V[0] = q;
-		w(g, "    int ~W0();~N");
-	    }
-	}
+        for (q = g->analyser->names; q; q = q->next) {
+            if (q->type == t_routine && !q->used_in_among) {
+                g->V[0] = q;
+                w(g, "    int ~W0();~N");
+            }
+        }
 
-	w(g, "~N  public:~N");
+        w(g, "~N  public:~N");
 
-	/* FIXME: We currently need to make any routines used in an among
-	 * public. */
-	for (q = g->analyser->names; q; q = q->next) {
-	    if (q->type == t_routine && q->used_in_among) {
-		g->V[0] = q;
-		w(g, "    int ~W0();~N");
-	    }
-	}
+        /* FIXME: We currently need to make any routines used in an among
+         * public. */
+        for (q = g->analyser->names; q; q = q->next) {
+            if (q->type == t_routine && q->used_in_among) {
+                g->V[0] = q;
+                w(g, "    int ~W0();~N");
+            }
+        }
 
-	w(g, "~N");
-	p = strrchr(g->options->name, ':');
-	if (p) ++p; else p = g->options->name;
-	g->S[1] = p;
-	w(g, "    ~S1();~N"
-	     "    ~~~S1();~N");
-	for (q = g->analyser->names; q; q = q->next) {
-	    if (q->type == t_external) {
-		g->V[0] = q;
-		w(g, "    int ~W0();~N");
-	    }
-	}
+        w(g, "~N");
+        p = strrchr(g->options->name, ':');
+        if (p) ++p; else p = g->options->name;
+        g->S[1] = p;
+        w(g, "    ~S1();~N"
+             "    ~~~S1();~N");
+        for (q = g->analyser->names; q; q = q->next) {
+            if (q->type == t_external) {
+                g->V[0] = q;
+                w(g, "    int ~W0();~N");
+            }
+        }
 
-	w(g, "    std::string get_description() const;~N"
-	     "};~N"
-	     "~N"
-	     "}~N");
+        w(g, "    std::string get_description() const;~N"
+             "};~N"
+             "~N"
+             "}~N");
 
-	return;
+        return;
     }
 
     w(g, "~N"
@@ -1859,15 +1903,29 @@ static void generate_header_file(struct generator * g) {
             case t_external:
                 w(g, "extern int ~W0(struct SN_env * z);~N");
                 break;
-            case t_string:  g->S[1] = "S"; goto label0;
-            case t_integer: g->S[1] = "I"; goto label0;
-            case t_boolean: g->S[1] = "B";
-            label0:
+            case t_string:
+            case t_integer:
+            case t_boolean:
                 if (vp) {
-                    g->I[0] = q->count;
+                    int count = q->count;
+                    if (count < 0) {
+                        /* Unused variables should get removed from `names`. */
+                        fprintf(stderr, "Optimised out variable ");
+                        report_b(stderr, q->b);
+                        fprintf(stderr, " still in names list\n");
+                        exit(1);
+                    }
+                    if (q->type == t_boolean) {
+                        /* We use a single array for booleans and integers,
+                         * with the integers first.
+                         */
+                        count += g->analyser->name_count[t_integer];
+                    }
+                    g->I[0] = count;
+                    g->I[1] = "SIIrxg"[q->type];
                     w(g, "#define ~S0");
-                    str_append_b(g->outbuf, q->b);
-                    w(g, " (~S1[~I0])~N");
+                    write_b(g, q->b);
+                    w(g, " (~c1[~I0])~N");
                 }
                 break;
         }
@@ -1884,19 +1942,19 @@ static void generate_header_file(struct generator * g) {
 extern void generate_program_c(struct generator * g) {
 
     g->outbuf = str_new();
-    generate_start_comment(g);
+    write_start_comment(g, "/* ", " */");
     generate_head(g);
     if (g->options->make_lang == LANG_C) {
-	generate_routine_headers(g);
-	w(g, "#ifdef __cplusplus~N"
-	     "extern \"C\" {~N"
-	     "#endif~N"
-	     "~N");
-	generate_create_and_close_templates(g);
-	w(g, "~N"
-	     "#ifdef __cplusplus~N"
-	     "}~N"
-	     "#endif~N");
+        generate_routine_headers(g);
+        w(g, "#ifdef __cplusplus~N"
+             "extern \"C\" {~N"
+             "#endif~N"
+             "~N");
+        generate_create_and_close_templates(g);
+        w(g, "~N"
+             "#ifdef __cplusplus~N"
+             "}~N"
+             "#endif~N");
     }
     generate_amongs(g);
     generate_groupings(g);
@@ -1914,7 +1972,7 @@ extern void generate_program_c(struct generator * g) {
     output_str(g->options->output_src, g->outbuf);
     str_clear(g->outbuf);
 
-    generate_start_comment(g);
+    write_start_comment(g, "/* ", " */");
     generate_header_file(g);
     output_str(g->options->output_h, g->outbuf);
     str_delete(g->outbuf);
