@@ -1756,8 +1756,18 @@ DEFINE_TESTCASE(unsupportedcheck1, path) {
 	<< ' ' << get_database_path("apitest_simpledata") << '\n';
     out.close();
 
-    TEST_EXCEPTION(Xapian::UnimplementedError,
-		   Xapian::Database::check(stubpath));
+    try {
+	Xapian::Database::check(stubpath);
+	FAIL_TEST("Managed to check remote stub");
+#ifdef XAPIAN_HAS_REMOTE_BACKEND
+    } catch (const Xapian::UnimplementedError& e) {
+	// Check the message is appropriate.
+	TEST_STRINGS_EQUAL(e.get_msg(),
+			   "Remote database checking not implemented");
+#else
+    } catch (const Xapian::FeatureUnavailableError& e) {
+#endif
+    }
 }
 
 // Test exception for check() on inmemory via stub.
