@@ -96,7 +96,7 @@ check_sparse_uid_terms(const string & path)
 }
 
 // With multi the docids in the shards change the behaviour.
-DEFINE_TESTCASE(compactoldnorenumber1, compact && generated && !multi) {
+DEFINE_TESTCASE(compactoldnorenumber1, compact && !multi) {
     string a = get_database_path("compactnorenumber1a", make_sparse_db,
 				 "5-7 24 76 987 1023-1027 9999 !9999");
     string a_uuid;
@@ -275,7 +275,7 @@ make_multichunk_db(Xapian::WritableDatabase &db, const string &)
 
 // Test use of compact on a database which has multiple chunks for a term.
 // This is a regression test for ticket #427
-DEFINE_TESTCASE(compactoldmultichunks1, compact && generated) {
+DEFINE_TESTCASE(compactoldmultichunks1, compact) {
     string indbpath = get_database_path("compactmultichunks1in",
 					make_multichunk_db, "");
     string outdbpath = get_compaction_output_path("compactmultichunks1out");
@@ -424,7 +424,7 @@ make_missing_tables(Xapian::WritableDatabase &db, const string &)
     db.commit();
 }
 
-DEFINE_TESTCASE(compactoldmissingtables1, compact && generated) {
+DEFINE_TESTCASE(compactoldmissingtables1, compact) {
     string a = get_database_path("compactmissingtables1a",
 				 make_all_tables);
     string b = get_database_path("compactmissingtables1b",
@@ -463,7 +463,7 @@ make_all_tables2(Xapian::WritableDatabase &db, const string &)
 }
 
 /// Adds coverage for merging synonym table.
-DEFINE_TESTCASE(compactoldmergesynonym1, compact && generated) {
+DEFINE_TESTCASE(compactoldmergesynonym1, compact) {
     string a = get_database_path("compactmergesynonym1a",
 				 make_all_tables);
     string b = get_database_path("compactmergesynonym1b",
@@ -538,7 +538,7 @@ DEFINE_TESTCASE(compactoldempty1, compact) {
     }
 }
 
-DEFINE_TESTCASE(compactoldmultipass1, compact && generated) {
+DEFINE_TESTCASE(compactoldmultipass1, compact) {
     string outdbpath = get_compaction_output_path("compactmultipass1");
     rm_rf(outdbpath);
 
