@@ -335,26 +335,6 @@ DEFINE_TESTCASE(stubdb8, inmemory) {
     }
 }
 
-/// Test error running Database::check() on a remote stub database.
-DEFINE_TESTCASE(stubdb9, path) {
-    mkdir(".stub", 0755);
-    const char * dbpath = ".stub/stubdb9";
-    ofstream out(dbpath);
-    TEST(out.is_open());
-    out << "remote :" << BackendManager::get_xapian_progsrv_command()
-	<< ' ' << get_database_path("apitest_simpledata") << '\n';
-    out.close();
-
-    try {
-	Xapian::Database::check(dbpath);
-	FAIL_TEST("Managed to check remote stub");
-    } catch (const Xapian::UnimplementedError& e) {
-	// Check the message is appropriate.
-	TEST_STRINGS_EQUAL(e.get_msg(),
-			   "Remote database checking not implemented");
-    }
-}
-
 #if 0 // the "force error" mechanism is no longer in place...
 class MyErrorHandler : public Xapian::ErrorHandler {
     public:
