@@ -1,7 +1,7 @@
 /** @file
  * @brief BackendManager subclass for singlefile databases.
  */
-/* Copyright (C) 2007,2008,2009,2011,2012,2013,2015,2018 Olly Betts
+/* Copyright (C) 2007,2008,2009,2011,2012,2013,2015,2018,2023 Olly Betts
  * Copyright (C) 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -33,7 +33,7 @@ using namespace std;
 
 BackendManagerSingleFile::BackendManagerSingleFile(const string& datadir_,
 						   BackendManager* sub_manager_)
-    : BackendManager(datadir_),
+    : BackendManager(datadir_, "singlefile_" + sub_manager_->get_dbtype()),
       sub_manager(sub_manager_),
       cachedir(".singlefile" + sub_manager_->get_dbtype())
 {
@@ -41,12 +41,6 @@ BackendManagerSingleFile::BackendManagerSingleFile(const string& datadir_,
     (void)create_dir_if_needed(cachedir);
 }
 
-std::string
-BackendManagerSingleFile::get_dbtype() const
-{
-    return "singlefile_" + sub_manager->get_dbtype();
-}
-
 string
 BackendManagerSingleFile::do_get_database_path(const vector<string> & files)
 {
@@ -63,19 +57,59 @@ BackendManagerSingleFile::do_get_database_path(const vector<string> & files)
 	sub_manager->get_database(files).compact(tmp_path,
 						 Xapian::DBCOMPACT_SINGLE_FILE);
 	if (rename(tmp_path.c_str(), db_path.c_str()) < 0) {
-	    throw Xapian::Database("rename failed", errno);
+	    throw Xapian::DatabaseError("rename failed", errno);
 	}
     }
 
     return db_path;
 }
 
+Xapian::WritableDatabase
+BackendManagerSingleFile::get_generated_database(const string& name)
+{
+    // Create generated database inside sub_manager's cache.
+    return sub_manager->get_generated_database(name);
+}
+
+void
+BackendManagerSingleFile::finalise_generated_database(const string& name)
+{
+    create_dir_if_needed(cachedir);
+
+    // path to the temporary generated db
+    string generated_db_path = sub_manager->get_generated_database_path(name);
+
+    // path to final singlefile db
+    string path = cachedir + "/" + name;
+
+    // path to tmpfile
+    string tmpfile = path + ".tmp";
+
+    // Convert to singlefile db.
+    {
+	Xapian::Database db(generated_db_path);
+	db.compact(tmpfile,
+		   Xapian::DBCOMPACT_SINGLE_FILE |
+		   Xapian::DBCOMPACT_NO_RENUMBER);
+    }
+
+    if (rename(tmpfile.c_str(), path.c_str()) < 0) {
+	throw Xapian::DatabaseError("rename failed", errno);
+    }
+}
+
 Xapian::WritableDatabase
 BackendManagerSingleFile::get_writable_database(const string &, const string &)
 {
     throw Xapian::UnimplementedError("Single-file databases don't support writing");
 }
 
+string
+BackendManagerSingleFile::get_generated_database_path(const string& name)
+{
+    return cachedir + "/" + name;
+}
+
 string
 BackendManagerSingleFile::get_compaction_output_path(const string& name)
 {
